program WinShebang;

{$APPTYPE CONSOLE}

{$IFDEF FPC}
  {$MODE Delphi}
{$ENDIF}

uses
  Classes, SysUtils, Windows {$IFDEF FPC}, StreamEx{$ENDIF};

type
  TStringArray = array of string;

procedure StringArrayAppend(var arr: TStringArray; const str: string);
begin
  SetLength(arr, Length(arr) + 1);
  arr[High(arr)] := str;
end;

procedure ExitWithError(const strError: string; iCode: Integer);
begin
  MessageBox(0, PChar(strError), 'WinShebang', 0);
  ExitProcess(iCode);
end;

procedure RunProcessAndWait(const args: TStringArray);
var
  PI: TProcessInformation;
  SI: TStartupInfo;
  hIn, hOut, hErr: THandle;
  bOk: Boolean;
  strCmdLine: string;
  i: Integer;
begin
  strCmdLine := '';
  for i := 0 to High(args) do
    strCmdLine := strCmdLine + args[i] + #32;

  MessageBox(0, PChar('Command line: ' + strCmdLine), 'WinShebang', 0);

  hIn := GetStdHandle(STD_INPUT_HANDLE);
  hOut := GetStdHandle(STD_OUTPUT_HANDLE);
  hErr := GetStdHandle(STD_ERROR_HANDLE);
  ZeroMemory(@PI, sizeof(PROCESS_INFORMATION));
  ZeroMemory(@SI, sizeof(STARTUPINFO));
  SI.cb := sizeof(STARTUPINFO);
  SI.hStdError := hErr;
  SI.hStdOutput := hOut;
  SI.hStdInput := hIn;
  SI.dwFlags := STARTF_USESTDHANDLES;

  bOK := CreateProcess(nil, PChar(strCmdLine),
    nil, nil, True, CREATE_UNICODE_ENVIRONMENT, nil, nil, SI, PI);

  if not bOK then
    ExitWithError('Failed to execute commandline: ' + strCmdLine, -1)
  else
    WaitForSingleObject(PI.hProcess, INFINITE);

end;

function GetShebangExeAndParams(const strFileName: string): TStringArray;
var
  s: TFileStream;
  r: TTextReader;
  str: string;
  lst: TStrings;
  i: Integer;
begin

  SetLength(Result, 0);

  if not FileExists(strFileName) then
    ExitWithError('File does not exist: ' + strFileName, -1)
  else
  begin

    str := '';

    s := TFileStream.Create(strFileName, fmOpenRead);
    r := TStreamReader.Create(s);
    try
      str := r.ReadLine();
    finally
      r.Free;
      s.Free;
    end;

    if Pos('#!', str) <> 1 then
      ExitWithError('File does not contain shebang string', -1)
    else
    begin
      Delete(str, 1, 2);
      lst := TStringList.Create;
      try
        lst.Text := StringReplace(str, #32, #$D#$A, [rfReplaceAll, rfIgnoreCase]);
        for i := 0 to lst.Count - 1 do
          if lst[i] <> '' then
            StringArrayAppend(Result, lst[i])
      finally
        lst.Free;
      end;
    end;

  end;

end;

function ParseArgs(): TStringArray;
var
  i: Integer;
begin
  Result := GetShebangExeAndParams(ParamStr(1));
  for i := 1 to ParamCount() do
    StringArrayAppend(Result, ParamStr(i));
end;

begin

  ExitCode := 0;

  if ParamCount() < 1 then
    ExitWithError('Usage: WinShebang.exe <shebang_file> [arg1]..[argn]', -1)
  else
    RunProcessAndWait(ParseArgs());

  MessageBox(0, 'Done', 'WinShebang', 0);

end.
